
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>

const long IE_COOKIE = 0x7F007F00;

typedef struct 
{
	long cookie;
	long msgno;
	long msg_plus_header_size;
	long true_msg_size;
} IEHeader;


int 
main( int argc, const char *argv[] )
{
	FILE	*fpin;
	FILE	*fpout;
	FILE	*fp;
	int		msg = 0;
	int		corrupted_msgs = 0;
	long	fsize;
	char	infile[32];
	char	outfile[32];
	char	*fdata;
	char	*fdp;
	char	*fend;
	IEHeader	*pIEHeader;
	/*
	 * determine infile and outfile
	 */

	// Zuwenig oder zuviel Argumente
	
	if ( argc != 2 )
	{
		puts( "Usage: m2e infile.ext" );
		puts( " outfiles = 1.txt, 2.txt, 3.txt ..." );
		exit( 1 );
	}
	
	// Bei Angabe nur der zu konvertierenden Datei wird automatisch eine
	// <Datei>.mbx erzeugt.

	strcpy( infile, argv[1] );
	
	// Datei lesegeschtzt?
	if ( ( fpin = fopen( infile, "rb" ) ) == 0 )
	{
		printf( "M2E: could not open '%s': readprotected\n", infile );
		exit( 1 );
	}

	/*
	 * read in the entire file
	 */

	//  Dateigre (Imput) und Speicherplatz anlegen
	fsize = _filelength( fileno( fpin ) );
	if ((fdata = ( char * ) malloc( fsize ) ) == NULL )
	{
		printf( "M2E: Couldn't allocate enough memory" );
		exit( 1 );
	}

	// Einlesen der gesammten Datei
	fdp = fdata;
	fend = fdata + fsize;

	if ( fread( fdata, fsize, 1, fpin ) != 1 )
	{
		printf( "M2E: could not read all of '%s'\n", infile );
		exit( 1 );
	}

	fclose( fpin );

	/*
	 * there are 84 bytes of magical binary header that I never
	 * bothered to decode, so we skip past them
	 */

	fdp += 84;

	while ( fdp < fend )
	{
		int msgsize;
		
		printf( "processing message %d\r", ++msg );
		pIEHeader = (IEHeader *) fdp;
		fdp += sizeof( IEHeader );

		sprintf( outfile, "%i.txt", msg);
		
		// Datei schreibgeschtzt?
		if ( ( fpout = fopen( outfile, "wb" ) ) == 0 )
		{
			fclose( fpin );
			printf( "M2E: could not open '%s': writeprotected\n", outfile );
			exit( 1 );
		}

		/*
		 * if we don't find the magic cookie, then we have encountered
		 * a serious problem and need to quit
		 */

		if 	( pIEHeader->cookie != IE_COOKIE )
		{
			corrupted_msgs++;
			fp = fopen( "m2e.log", "a+t" );

			if ( fp )
			{
				fprintf( fp, "M2E: file '%s' is corrupted, message %d\n", infile, msg );
				fclose( fp );
			}
			printf( "M2E: file '%s' is corrupted, aborting", infile);
			exit( 1 );
		}

		msgsize = pIEHeader->msg_plus_header_size - sizeof( IEHeader);

		fwrite( fdp, msgsize, 1, fpout );

		if ( fdp[msgsize-1] != '\n' )
			fprintf( fpout, "\n" );

		fdp += msgsize;

		// Output-Datei schlieen
		fclose( fpout );

		/* 
		 * look for the next header, which should be at fdp
		 */

		if ( fdp < fend )
		{
			pIEHeader = ( IEHeader * ) fdp;
			if ( pIEHeader->cookie != IE_COOKIE )
			{
				/*
				 * corrupted message!!!
				 */

				fdp -= msgsize;

				while ( fdp < fend-sizeof( IE_COOKIE ) )
				{
					if ( memcmp( fdp, &IE_COOKIE, sizeof( IE_COOKIE ) ) == 0 )
					{
						break;
					} // end if

					fdp++;
				} // end while

				if ( fdp > fend-sizeof( IE_COOKIE ) )
					break;
			} // end if

		} // end if

	}  // end while

	free( fdata );

	fclose( fpout );

	/*
	 * print a summary
	 */

	printf( "\n\nM2E Summary\n" );
	printf( "Input: %s\n", infile );
	printf( "Output: %s\n", outfile );
	printf( "Messages: %d\n", msg );
	printf( "Corrupted Messages: %d\n", corrupted_msgs );
	return 0;

} // end main




