
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>

const long IE_COOKIE = 0x7F007F00;

const char eudora_header[] = "From ???@??? Sun Jan 00 00:00:00 0000";

struct IEHeader
{
	long cookie;
	long msgno;
	long msg_plus_header_size;
	long true_msg_size;
};

static IEHeader *pIEHeader;

static long fsize;

static char infile[512];
static char outfile[512];
static char *fdata, *fdp, *fend;

int strnstr( const char *s, const char *substring, int n )
{
	char *p = ( char * ) malloc( n + 1 );
	int found = 0;
	if ( p )
	{
		memcpy( p, s, n );
		p[n] = 0;
		found = strstr( p, substring ) ? 1 : 0;
		free( p );
	}
	else
	{
		puts( "M2E: Out of memory" );
		exit( 1 );
	}
	return found;
}


int 
main( int argc, const char *argv[] )
{
	FILE *fpin, *fpout;
	int msg = 0, corrupted_msgs = 0;

/*
 *
 * determine infile and outfile
 *
 */

if ( argc == 2 )
{
	strcpy( infile, argv[1] );
	strcpy( outfile, infile );

	if ( strchr( outfile, '.' ) )
	*strchr( outfile, '.' ) = 0;

	strcat( outfile, ".mbx" );
}
else 	
	if ( argc == 3 )
	{
		strcpy( infile, argv[1] );
		strcpy( outfile, argv[2] );
	}
	else
	{
		puts( "Usage: m2e infile.ext [outfile.ext]" );
		puts( " by default outfile = infile.mbx" );
		exit( 1 );
	}

/*
 *
 * make sure files aren't the same
 *
 */

if ( _stricmp( infile, outfile ) == 0 )
{
	printf( "Having the same input and output file is NOT a good idea!\n" );
	exit( 1 );
}


if ( ( fpin = fopen( infile, "rb" ) ) == 0 )
{
	printf( "M2E: could not open '%s'\n", infile );
	exit( 1 );
}

if ( ( fpout = fopen( outfile, "wb" ) ) == 0 )
{
	fclose( fpin );
	printf( "M2E: could not open '%s'\n", outfile );
	exit( 1 );
}


/*


** read in the entire file


*/


	fsize = _filelength( fileno( fpin ) );
	fdata = ( char * ) malloc( fsize );
	
	fdp = fdata;
	fend = fdata + fsize;

if ( fread( fdata, fsize, 1, fpin ) != 1 )
{
	printf( "M2E: could not read all of '%s'\n", infile );
	exit( 1 );
}
fclose( fpin );




/*
 *
 * there are 84 bytes of magical binary header that I never
 * bothered
 * to decode, so we skip past them
 *
 */


fdp += 84;

while ( fdp < fend )
{
	int msgsize;
	
	printf( "processing message %d\r", ++msg );
	pIEHeader = ( IEHeader * ) fdp;
	fdp += sizeof( IEHeader );

/*
 * if we don't find the magic cookie, then we have encountered
 * a serious problem and need to quit
 *
 */

if 	( pIEHeader->cookie != IE_COOKIE )
{
	corrupted_msgs++;
	FILE *fp = fopen( "m2e.log", "a+t" );

	if ( fp )
	{
		fprintf( fp, "M2E: file '%s' is corrupted, message %d\n", infile, msg );
		fclose( fp );
	}
	printf( "M2E: file '%s' is corrupted, aborting", infile);
	exit( 1 );
}
else
{
	msgsize = pIEHeader->msg_plus_header_size - sizeof( IEHeader);
	if ( !strnstr( fdp, "From ???@???", msgsize ) )
		fprintf( fpout, "%s\n", eudora_header );

	fwrite( fdp, msgsize, 1, fpout );

	if ( fdp[msgsize-1] != '\n' )
		fprintf( fpout, "\n" );

	fdp += msgsize;
}




/* 
 *
 * look for the next header, which should be at fdp
 *
 */

if ( fdp < fend )
{
	pIEHeader = ( IEHeader * ) fdp;
	if ( pIEHeader->cookie != IE_COOKIE )
	{

	/*
	 * corrupted message!!!
	 *
	 */

	fdp -= msgsize;

	while ( fdp < fend-sizeof( IE_COOKIE ) )
	{
		if ( memcmp( fdp, &IE_COOKIE, sizeof( IE_COOKIE ) ) == 0 )
		{
			break;
		}

		fdp++;

	}


	if ( fdp > fend-sizeof( IE_COOKIE ) )
		break;
} // end if


}


}




free( fdata );


fclose( fpout );




/*


** print a summary


*/


printf( "\n\nM2E Summary\n" );
printf( "Input: %s\n", infile );
printf( "Output: %s\n", outfile );
printf( "Messages: %d\n", msg );
printf( "Corrupted Messages: %d\n", corrupted_msgs );
return 0;


}




